package cloud.linktime.market;

import cloud.linktime.market.common.Constant;
import cloud.linktime.market.common.DateUtil;
import cloud.linktime.market.common.RedisUtil;
import cloud.linktime.market.common.Util;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;

/**
 * Created by anna on 2019/6/13.
 */
public class ProduceDataToRedis {
  private Logger logger = Logger.getLogger(ProduceDataToRedis.class);

  private static Map<String, String> goodsData = new HashMap<>();
  private static Map<String, String> storeData = new HashMap<>();
  private static Map<String, Integer> sChannel = new HashMap<>();
  private static Map<String, Integer> sPayment = new HashMap<>();

  public String selectOne(Map<String, Integer> map) {
    Integer sum = 0;
    Set<Map.Entry<String, Integer>> entrySet = map.entrySet();
    Iterator<Map.Entry<String, Integer>> iterator = entrySet.iterator();
    while (iterator.hasNext()) {
      sum += iterator.next().getValue();
    }
    Integer rand = new Random().nextInt(sum) + 1;
    for (Map.Entry<String, Integer> entry : map.entrySet()) {
      rand -= entry.getValue();
      if (rand <= 0) {
        return entry.getKey();
      }
    }
    return "其他";
  }

  public String getRandomStoreId() {
    Random ra = new Random();
    int storeSize = storeData.size();
    String storeId = new ArrayList<String>(storeData.keySet()).get(ra.nextInt(storeSize));
    return storeId;
  }

  public String getRandomGoodsId() {
    Random ra = new Random();
    int goodsSize = goodsData.size();
    String goodsId = new ArrayList<String>(goodsData.keySet()).get(ra.nextInt(goodsSize));
    return goodsId;
  }

  public void generateInventoryData() {
    String inventoryKey = Util.getRedisInventoryKey();
    int inventory = Constant.DEFAULT_INVENTORY;
    String goodsStoreId = getRandomGoodsId() + Constant.SPLIT + getRandomStoreId();
    Random ra = new Random();
    inventory =  ra.nextInt(inventory) +100;
    RedisUtil.hset(inventoryKey, goodsStoreId, String.valueOf(inventory), false);
  }

  public void generateOrderData() {
    String payment = selectOne(sPayment);
    String saleChannel = selectOne(sChannel);
    Random ra = new Random();
    long datetime = System.currentTimeMillis();
    String goodsId = getRandomGoodsId();
    String storeId = getRandomStoreId();
    String day = DateUtil.getStringDate(new Date(datetime), DateUtil.DEFAULT_DATE_FORMAT);
    String area = RedisUtil.getArea(storeId);
    String orderKey = Util.getRedisOrderDay() + "_" + area + "_" + day;
    String paymentKey = Util.getRedisPaymentDay() + "_" + day;
    String[] keyOrder = {goodsId, storeId};
    String[] keyPayment = {payment, saleChannel};
    String keyOrderString = StringUtils.join(keyOrder, Constant.SPLIT);
    String valueOrder = RedisUtil.hget(orderKey, keyOrderString);
    long tradeNum = ra.nextInt(15) + 1L;
    long countNum = 1L;
    BigDecimal paid = new BigDecimal(ra.nextInt(100) + 1);
    String[] values = {paid.toString(), String.valueOf(tradeNum), String.valueOf(countNum)};
    if (null == valueOrder) {
      RedisUtil.hset(orderKey, keyOrderString, StringUtils.join(values, Constant.SPLIT), true);
    } else {
      RedisOrderValue redisOrderValue = new RedisOrderValue(valueOrder);
      BigDecimal newPaid = paid.add(redisOrderValue.getPaid());
      long newTradeNum = tradeNum + redisOrderValue.getOrderNum();
      long newCountNum = countNum + redisOrderValue.getCount();
      String[] newValues = {newPaid.toString(), String.valueOf(newTradeNum), String.valueOf(newCountNum)};
      RedisUtil.hset(orderKey, keyOrderString, StringUtils.join(newValues, Constant.SPLIT), false);
    }
    String keyPaymentString = StringUtils.join(keyPayment, Constant.SPLIT);
    String valuePayment = RedisUtil.hget(paymentKey, keyPaymentString);
    if (null == valuePayment) {
      RedisUtil.hset(paymentKey, keyPaymentString, StringUtils.join(values, Constant.SPLIT), true);
    } else {
      RedisOrderValue redisOrderValue = new RedisOrderValue(valuePayment);
      BigDecimal newPaid = paid.add(redisOrderValue.getPaid());
      long newTradeNum = tradeNum + redisOrderValue.getOrderNum();
      long newCountNum = countNum + redisOrderValue.getCount();
      String[] newValues = {newPaid.toString(), String.valueOf(newTradeNum), String.valueOf(newCountNum)};
      RedisUtil.hset(paymentKey, keyPaymentString, StringUtils.join(newValues, Constant.SPLIT), false);
    }
  }

  public void generateStoreMinuteData() {
    String redisStoreAmountCountMinute = Util.getResidStoreAmountCountMinute();
    String storekey = redisStoreAmountCountMinute + "_" + getRandomStoreId();
    Map<String, String> map = new HashMap();
    Random ra = new Random();
    Integer amount = ra.nextInt(10000) + 100;
    Integer num = ra.nextInt(100) + 10;
    map.put("amount", String.valueOf(amount));
    map.put("num", String.valueOf(num));
    RedisUtil.hmset(storekey, map, true);
  }

  public void send2kafka() {
    RedisUtil.baseInfo2Redis();
    try {
      goodsData = RedisUtil.getGoodsData();
      storeData = RedisUtil.getStoreData();
      if (goodsData == null) {
        logger.error("can not read goods data from redis");
      }
      if (storeData == null) {
        logger.error("can not read store data from redis");
      }
      String zhifuBao = System.getenv("zhifubao");
      String weixin = System.getenv("weixin");
      String xianjin = System.getenv("xianjin");
      String yinlian = System.getenv("yinlian");
      int zhifubaoInt = 4;
      if (null != zhifuBao) {
        zhifubaoInt = Integer.parseInt(zhifuBao);
      }
      int weixinInt = 4;
      if (null != weixin) {
        weixinInt = Integer.parseInt(weixin);
      }
      int xianjinInt = 1;
      if (null != xianjin) {
        xianjinInt = Integer.parseInt(xianjin);
      }
      int yinlianInt = 1;
      if (null != yinlian) {
        yinlianInt = Integer.parseInt(yinlian);
      }
      sChannel.put("线下门店", 5);
      sChannel.put("京东", 2);
      sChannel.put("天猫", 2);
      sChannel.put("微信商城", 1);
      sPayment.put("支付宝", zhifubaoInt);
      sPayment.put("微信支付", weixinInt);
      sPayment.put("现金支付", xianjinInt);
      sPayment.put("银联支付", yinlianInt);
      while (true) {
        generateOrderData();
        generateStoreMinuteData();
        generateInventoryData();
        try {
          Thread.sleep(200);
          logger.info("sheep 200s.");
        } catch (InterruptedException e) {
          logger.error("thread sleep error.");
        }
      }
    } catch (RuntimeException e) {
      logger.error("send message to redis failed.", e);
    }
  }

  public static void main(String[] args) {
    new ProduceDataToRedis().send2kafka();
  }

}
