package cloud.linktime.market.common;

import org.apache.log4j.Logger;

import cloud.linktime.market.Goods;
import cloud.linktime.market.Store;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;
/**
 * Created by anna on 2019/6/13.
 */
public class RedisUtil {
  private static Logger logger = Logger.getLogger(RedisUtil.class);
  private static Map<String, String> goodsData = new HashMap<>();
  private static Map<String, String> storeData = new HashMap<>();
  private static Map<String, String> inventoryData = new HashMap<>();
  private static Map<String, Store> storeMap = new HashMap<>();
  private static Map<String, Goods> goodsMap = new HashMap<>();
  private static long timeinterval = 24 * 60 * 60 * 1000L;
  private static long lastUpdate = System.currentTimeMillis();
  private final static byte[] lockGoods = new byte[0];
  private final static byte[] lockStore = new byte[0];
  public static JedisPool redisPool = null;

  static {
    String redisServer = Util.getRedisServer();
    String[] redisServers = redisServer.split(":");
    JedisPoolConfig config = new JedisPoolConfig();
    config.setMaxTotal(200);
    config.setMaxIdle(200);
    config.setMaxWaitMillis(5000);
    config.setTestOnBorrow(true);
    redisPool = new JedisPool(config, redisServers[0], Integer.valueOf(redisServers[1]), 5000);
  }


  public static void generateStoreOrGoods(String filename) {
    BufferedReader reader = null;

      InputStream inStream;
      inStream = RedisUtil.class.getClassLoader().getResourceAsStream(filename);
      reader = new BufferedReader(new InputStreamReader(inStream));

    String line = "";
    try {
      while ((line = reader.readLine()) != null) {
        String[] ops = line.split(",");
        if (ops.length == 5) {
          String goodsId = ops[0];
          goodsData.put(goodsId, line);
        }
        if (ops.length == 7) {
          String storeId = ops[0];
          storeData.put(storeId, line);
        }
      }
    } catch (IOException e) {
      logger.error("generate basic data failed: " + filename, e);
    }
  }

  public static void hmset(String redisKey, Map<String, String> map, Boolean expire) {

    Jedis jedis = null;
    try {
      jedis = redisPool.getResource();
      jedis.hmset(redisKey, map);
      if (expire){
        jedis.expire(redisKey, 60);
      }
    } catch (Exception e) {
      logger.error(redisKey + ": hmset has error", e);
    } finally {
      if (jedis != null) {
        jedis.close();
      }
    }
  }

  public static Map<String, String> hgetAll(String redisKey) {
    Jedis jedis = null;
    Map<String, String> redisContent = null;
    try {
      jedis = redisPool.getResource();
      redisContent = jedis.hgetAll(redisKey);
    } catch (Exception e) {
      logger.error(redisKey + ": hgetAll has error", e);
    } finally {
      if (jedis != null) {
        jedis.close();
      }
    }
    return redisContent;
  }

  public static String hget(String redisKey, String id) {
    Jedis jedis = null;
    String value = null;
    try {
      jedis = redisPool.getResource();
      value = jedis.hget(redisKey, id);
    } catch (Exception e) {
      logger.error(redisKey + ": hget has error", e);
    } finally {
      if (jedis != null) {
        jedis.close();
      }
    }
    return value;
  }

  public static void hset(String redisKey, String id, String value, Boolean expire) {
    Jedis jedis = null;
    try {
      jedis = redisPool.getResource();
      jedis.hset(redisKey, id, value);
      if (expire){
        jedis.expire(redisKey, 60*60*24*2);
      }
    } catch (Exception e) {
      logger.error(redisKey + ": hset has error", e);
    } finally {
      if (jedis != null) {
        jedis.close();
      }
    }

  }

  public static void generateInventory() {
    if (storeData.size() > 0 && goodsData.size() > 0) {
      for (String goodsId : goodsData.keySet()) {
        for (String storeId : storeData.keySet()) {
          String goodsStoreId = goodsId + "," + storeId;
          inventoryData.put(goodsStoreId, String.valueOf(Constant.DEFAULT_INVENTORY));
        }
      }
    }
  }

  public static Map<String, String> getGoodsData(){
    return goodsData;
  }

  public static Map<String, String> getStoreData(){
    return storeData;
  }

  public static void baseInfo2Redis() {
    String goodsKey = Util.getRedisGoodsKey();
    String storeKey = Util.getRedisStoreKey();
    String inventoryKey = Util.getRedisInventoryKey();
    if (goodsData.size() == 0) {
      generateStoreOrGoods("goodsFile");
      hmset(goodsKey, goodsData, false);
    }
    if (storeData.size() == 0) {
      generateStoreOrGoods("storeFile");
      hmset(storeKey, storeData, false);
    }
    if (inventoryData.size() == 0) {
      generateInventory();
      hmset(inventoryKey, inventoryData, false);
    }
  }

  public static Map<String, Goods> getGoodsMap() {
    if (goodsData.size() == 0) {
      generateStoreOrGoods("goodsFile");
    }
    long currentTime = System.currentTimeMillis();
    if ((currentTime - lastUpdate) <= timeinterval && goodsMap.size() != 0) {
      return goodsMap;
    } else {
      synchronized (lockGoods) {
        if ((currentTime - lastUpdate) > timeinterval || goodsMap.size()==0) {
          Map<String, Goods> gmap = new HashMap<>();
          Map<String, String> map = goodsData;
          for (Map.Entry<String, String> entry : map.entrySet()) {
            String value = entry.getValue();
            String[] valueList = value.split(",");
            if (valueList.length == 5) {
              String goodsId = valueList[0];
              String goodsName = valueList[1];
              String category = valueList[2];
              String brand = valueList[3];
              BigDecimal price = new BigDecimal(valueList[4]);
              Goods goods = new Goods(goodsId, goodsName, category, brand, price);
              gmap.put(goodsId, goods);
            }
          }
          if (gmap.size() != 0) {
            goodsMap = gmap;
            lastUpdate = currentTime;
          }
        }
      }
      return goodsMap;
    }
  }

  public static Map<String, Store> getStoreMap() {
    if (storeData.size() == 0) {
      generateStoreOrGoods("storeFile");
    }
    long currentTime = System.currentTimeMillis();
    if ((currentTime - lastUpdate) <= timeinterval && storeMap.size() != 0) {
      return storeMap;
    } else {
      synchronized (lockStore) {
        if ((currentTime - lastUpdate) > timeinterval || storeMap.size()==0) {
          Map<String, String> map = storeData;
          Map<String, Store> smap = new HashMap<>();
          for (Map.Entry<String, String> entry : map.entrySet()) {
            String value = entry.getValue();
            String[] valueList = value.split(",");
            if (valueList.length == 7) {
              String storeId = valueList[0];
              String storeName = valueList[1];
              String province = valueList[2];
              String city = valueList[3];
              String area = valueList[4];
              String longitude = valueList[5];
              String latitude = valueList[6];
              Store store = new Store(storeId, storeName, province, city, area, longitude, latitude);
              smap.put(storeId, store);
            }
          }
          if (smap.size() != 0) {
            storeMap = smap;
            lastUpdate = currentTime;
          }
        }
      }
      return storeMap;
    }
  }

  public static String getArea(String orderId){
    Map<String, Store> map= getStoreMap();
    return map.get(orderId).getArea();
  }

  public static void main(String[] args) {

  }
}
